@@ -33,6 +33,22 @@ static QString titleFromItem(const Akonadi::Item &item)
     }
 }
 
+static QStringList titlesFromItems(const QList<Akonadi::Item> &items)
+{
+    auto result = QStringList();
+    std::transform(items.constBegin(), items.constEnd(), std::back_inserter(result), titleFromItem);
+    result.sort();
+    return result;
+}
+
+static QStringList displayNamesFromCollections(const QList<Akonadi::Collection> &collections)
+{
+    auto result = QStringList();
+    std::transform(collections.constBegin(), collections.constEnd(), std::back_inserter(result), std::bind(&Akonadi::Collection::displayName, _1));
+    result.sort();
+    return result;
+}
+
 class AkonadiLiveQueryHelpersTest : public QObject
 {
     Q_OBJECT
@@ -87,18 +103,21 @@ private slots:
         fetch(add);
         TestHelpers::waitForEmptyJobQueue();
 
-        auto result = QStringList();
-        std::transform(collections.constBegin(), collections.constEnd(),
-                       std::back_inserter(result),
-                       std::bind(&Akonadi::Collection::displayName, _1));
-        result.sort();
-
         // THEN
         auto expected = QStringList();
         expected << QStringLiteral("43") << QStringLiteral("46") << QStringLiteral("49") << QStringLiteral("52");
 
         expected.sort();
-        QCOMPARE(result, expected);
+        QCOMPARE(displayNamesFromCollections(collections), expected);
+
+        // WHEN (fetching twice in a row)
+        collections.clear();
+        fetch(add);
+        fetch(add);
+        TestHelpers::waitForEmptyJobQueue();
+
+        // THEN (the list shouldn't be duplicated)
+        QCOMPARE(displayNamesFromCollections(collections), expected);
 
         // WHEN (should not crash when the helpers object is deleted)
         helpers.clear();
@@ -107,12 +126,7 @@ private slots:
         TestHelpers::waitForEmptyJobQueue();
 
         // THEN
-        result.clear();
-        std::transform(collections.constBegin(), collections.constEnd(),
-                       std::back_inserter(result),
-                       std::bind(&Akonadi::Collection::displayName, _1));
-        result.sort();
-        QCOMPARE(result, expected);
+        QCOMPARE(displayNamesFromCollections(collections), expected);
     }
 
     void shouldFetchCollectionsForRoot_data()
@@ -159,12 +173,6 @@ private slots:
         fetch(add);
         TestHelpers::waitForEmptyJobQueue();
 
-        auto result = QStringList();
-        std::transform(collections.constBegin(), collections.constEnd(),
-                       std::back_inserter(result),
-                       std::bind(&Akonadi::Collection::displayName, _1));
-        result.sort();
-
         // THEN
         auto expected = QStringList();
 
@@ -181,7 +189,16 @@ private slots:
         }
 
         expected.sort();
-        QCOMPARE(result, expected);
+        QCOMPARE(displayNamesFromCollections(collections), expected);
+
+        // WHEN (fetching twice in a row)
+        collections.clear();
+        fetch(add);
+        fetch(add);
+        TestHelpers::waitForEmptyJobQueue();
+
+        // THEN (the list shouldn't be duplicated)
+        QCOMPARE(displayNamesFromCollections(collections), expected);
 
         // WHEN (should not crash when the helpers object is deleted)
         helpers.clear();
@@ -190,12 +207,7 @@ private slots:
         TestHelpers::waitForEmptyJobQueue();
 
         // THEN
-        result.clear();
-        std::transform(collections.constBegin(), collections.constEnd(),
-                       std::back_inserter(result),
-                       std::bind(&Akonadi::Collection::displayName, _1));
-        result.sort();
-        QCOMPARE(result, expected);
+        QCOMPARE(displayNamesFromCollections(collections), expected);
     }
 
     void shouldFetchItems()
@@ -236,17 +248,11 @@ private slots:
         fetch(add);
         TestHelpers::waitForEmptyJobQueue();
 
-        auto result = QStringList();
-        std::transform(items.constBegin(), items.constEnd(),
-                       std::back_inserter(result),
-                       titleFromItem);
-        result.sort();
-
         // THEN
         auto expected = QStringList();
         expected << QStringLiteral("43") << QStringLiteral("44") << QStringLiteral("47") << QStringLiteral("48");
         expected.sort();
-        QCOMPARE(result, expected);
+        QCOMPARE(titlesFromItems(items), expected);
 
         // WHEN (should not crash when the helpers object is deleted)
         helpers.clear();
@@ -255,12 +261,17 @@ private slots:
         TestHelpers::waitForEmptyJobQueue();
 
         // THEN
-        result.clear();
-        std::transform(items.constBegin(), items.constEnd(),
-                       std::back_inserter(result),
-                       titleFromItem);
-        result.sort();
-        QCOMPARE(result, expected);
+        QCOMPARE(titlesFromItems(items), expected);
+
+        // WHEN (fetching twice in a row)
+        helpers = createHelpers(data);
+        items.clear();
+        fetch(add);
+        fetch(add);
+        TestHelpers::waitForEmptyJobQueue();
+
+        // THEN (the list shouldn't be duplicated)
+        QCOMPARE(titlesFromItems(items), expected);
     }
 
     void shouldFetchItemsByCollection_data()
@@ -299,12 +310,6 @@ private slots:
         fetch(add);
         TestHelpers::waitForEmptyJobQueue();
 
-        auto result = QStringList();
-        std::transform(items.constBegin(), items.constEnd(),
-                       std::back_inserter(result),
-                       titleFromItem);
-        result.sort();
-
         // THEN
         auto expected = QStringList();
 
@@ -319,7 +324,16 @@ private slots:
         QVERIFY(!expected.isEmpty());
 
         expected.sort();
-        QCOMPARE(result, expected);
+        QCOMPARE(titlesFromItems(items), expected);
+
+        // WHEN (fetching twice in a row)
+        items.clear();
+        fetch(add);
+        fetch(add);
+        TestHelpers::waitForEmptyJobQueue();
+
+        // THEN (the list shouldn't be duplicated)
+        QCOMPARE(titlesFromItems(items), expected);
 
         // WHEN (should not crash when the helpers object is deleted)
         helpers.clear();
@@ -328,12 +342,7 @@ private slots:
         TestHelpers::waitForEmptyJobQueue();
 
         // THEN
-        result.clear();
-        std::transform(items.constBegin(), items.constEnd(),
-                       std::back_inserter(result),
-                       titleFromItem);
-        result.sort();
-        QCOMPARE(result, expected);
+        QCOMPARE(titlesFromItems(items), expected);
     }
 
     void shouldFetchItemsForContext_data()
@@ -386,12 +395,6 @@ private slots:
         fetch(add);
         TestHelpers::waitForEmptyJobQueue();
 
-        auto result = QStringList();
-        std::transform(items.constBegin(), items.constEnd(),
-                       std::back_inserter(result),
-                       titleFromItem);
-        result.sort();
-
         // THEN
         auto expected = QStringList();
 
@@ -402,7 +405,16 @@ private slots:
         QVERIFY(!expected.isEmpty());
 
         expected.sort();
-        QCOMPARE(result, expected);
+        QCOMPARE(titlesFromItems(items), expected);
+
+        // WHEN (fetching twice in a row)
+        items.clear();
+        fetch(add);
+        fetch(add);
+        TestHelpers::waitForEmptyJobQueue();
+
+        // THEN (the list shouldn't be duplicated)
+        QCOMPARE(titlesFromItems(items), expected);
 
         // WHEN (should not crash when the helpers object is deleted)
         helpers.clear();
@@ -411,12 +423,7 @@ private slots:
         TestHelpers::waitForEmptyJobQueue();
 
         // THEN
-        result.clear();
-        std::transform(items.constBegin(), items.constEnd(),
-                       std::back_inserter(result),
-                       titleFromItem);
-        result.sort();
-        QCOMPARE(result, expected);
+        QCOMPARE(titlesFromItems(items), expected);
     }
 
     void shouldFetchSiblings_data()
@@ -459,12 +466,6 @@ private slots:
         fetch(add);
         TestHelpers::waitForEmptyJobQueue();
 
-        auto result = QStringList();
-        std::transform(items.constBegin(), items.constEnd(),
-                       std::back_inserter(result),
-                       titleFromItem);
-        result.sort();
-
         // THEN
         auto expected = QStringList();
 
@@ -479,7 +480,16 @@ private slots:
         QVERIFY(!expected.isEmpty());
 
         expected.sort();
-        QCOMPARE(result, expected);
+        QCOMPARE(titlesFromItems(items), expected);
+
+        // WHEN (fetching twice in a row)
+        items.clear();
+        fetch(add);
+        fetch(add);
+        TestHelpers::waitForEmptyJobQueue();
+
+        // THEN (the list shouldn't be duplicated)
+        QCOMPARE(titlesFromItems(items), expected);
 
         // WHEN (should not crash when the helpers object is deleted)
         helpers.clear();
@@ -488,15 +498,12 @@ private slots:
         TestHelpers::waitForEmptyJobQueue();
 
         // THEN
-        result.clear();
-        std::transform(items.constBegin(), items.constEnd(),
-                       std::back_inserter(result),
-                       titleFromItem);
-        result.sort();
-        QCOMPARE(result, expected);
+        QCOMPARE(titlesFromItems(items), expected);
     }
 };
 
+// TODO: missing a test for LiveQueryHelpers::fetchTaskAndAncestors
+
 ZANSHIN_TEST_MAIN(AkonadiLiveQueryHelpersTest)
 
 #include "akonadilivequeryhelperstest.moc"
